import numpy as np
import xarray as xr


def symmetrize_xspectrum(xs, dim_range='k_rg', dim_azimuth='k_az'):
    """
    Symmetrize half-xspectrum around origin point. xspectrum is assumed to contain only positive wave-numbers in range.

    Args:
        xs (xarray.DataArray or xarray.Dataset): complex xspectra to be symmetrized
    Return:
        (xarray.DataArray or xarray.Dataset): symmetrized spectra (as they were generated in the first place using fft)
    """
    if (dim_range not in xs.coords) or (dim_azimuth not in xs.coords):
        raise ValueError(
            'Symmetry can not be applied because {} or {} do not have coordinates. Swap dimensions prior to symmetrization.'.format(
                dim_range, dim_azimuth))

    if not xs.sizes['k_az'] % 2:
        xs = xr.concat([xs, xs[{'k_az': 0}].assign_coords({'k_az': -xs.k_az[0].data})], dim='k_az')

    mirror = np.conj(xs[{dim_range: slice(None, 0, -1)}])
    mirror = mirror.assign_coords({dim_range: -mirror[dim_range], dim_azimuth: -mirror[dim_azimuth]})
    res = xr.concat([mirror, xs], dim=dim_range)[{dim_range: slice(None, -1), dim_azimuth: slice(None, -1)}]
    return res



# coding=utf-8
"""
purpose: file to share information about S-1 mission used in many scripts
Author: Antoine Grouazel
"""
import datetime
import os
import logging

PROJECT_DIR_DATARMOR = '/home/datawork-cersat-public/cache/project/mpc-sentinel1/'
PROJECT_DIR_DATARMOR_ALT = '/home/datawork-cersat-public/project/mpc-sentinel1/'
datarmor_archive_esa_ifremer = os.path.join(PROJECT_DIR_DATARMOR_ALT, "data", "esa")
# UNTARED_SPOOL = os.path.join(PROJECT_DIR,'workspace','spool_untared')
data_dir_s1a = os.path.join(PROJECT_DIR_DATARMOR, 'data', 'esa', 'sentinel-1a')
data_dir_s1b = os.path.join(PROJECT_DIR_DATARMOR, 'data', 'esa', 'sentinel-1b')
dir_aux_as_it_is_download = os.path.join(PROJECT_DIR_DATARMOR_ALT, 'data', 'auxiliary')
dir_data = {'S1A': data_dir_s1a,
            'S1B': data_dir_s1b}
ancillary_dataset = os.path.join(PROJECT_DIR_DATARMOR, "data", "ancillary")
mission_start_s1a = datetime.datetime(2014, 4, 6)
mission_start_s1b = datetime.datetime(2016, 4, 25)  # "first iw received
# Avant que S1B rejoigne son orbite definitive, les cycles ne faisaient pas 12 jours.
# Le premier cycle complet est le 10 et a commence le 14/06/2017 vers 18.00 apres l'orbite 152.
start_missions = {
    'S1A': mission_start_s1a,
    'S1B': mission_start_s1b}
first_s1b_wv = datetime.datetime(2016, 6, 15)  # WV valid Level2
first_s1a_wv = datetime.datetime(2015, 1,
                                 30)  # WV valid Level2 changed by agrouaze the 4dec 2018 (before was set to first acquisition Level1 WV)
first_valid_data = {'S1A': first_s1a_wv,
                    'S1B': first_s1b_wv}
s1a_calibration_date_WV = datetime.datetime(2014, 12, 12)
s1a_good_data_wv_calibrated = datetime.datetime(2016, 4, 18)


TYPES = ["OCN_", "GRDH", "SLC_", "GRDM", "GRDF", "RAW_"]
WORKING_DIR = os.path.join(PROJECT_DIR_DATARMOR_ALT, 'workspace')
sats_acro = {'S1A': 'sentinel-1a'
    , 'S1B': 'sentinel-1b'}
sats_full = {'sentinel-1a': 'S1A',
             'sentinel-1b': 'S1B'}
MODES = ['s1', 's2', 's3', 's4', 's5', 's6', 'wv', 'ew', 'iw']
macro_MODES = ['SM', 'WV', 'IW', 'EW']



def give_me_level_from_type(type_format):
    """
    type_format (str): ex: GRDH
    """
    if 'RAW' in type_format:
        level = 'L0'
    elif 'OCN' in type_format:
        level = 'L2'
    else:
        level = 'L1'
    return level


def give_me_prod_uploaded_at_pebsco(write=False):
    list_prod = []
    for sat in ['S1A', 'S1B']:
        #     for tprod in ['GRDH_1S','GRDF_1S','GRDM_1S','SLC__1S']:
        for tprod in ['GRDH_1S']:
            for mode in ['SM', 'IW', 'EW']:
                #         for mode in ['SM','IW','EW','WV']:
                list_prod.append(sat + '_' + mode + '_' + tprod)
    list_prod.append('S1A_WV_SLC__1S')
    list_prod.append('S1B_WV_SLC__1S')
    if write:
        filout = os.path.join(PROJECT_DIR_DATARMOR, "workspace", "syntool_upload_pebso/config_upload.txt")
        fid = open(filout, 'w')
        for dd in list_prod:
            fid.write(dd + '\n')
        fid.close()
        logging.info(filout)
    return list_prod


QUARANTINE = {
    # 'mpc':os.path.join(PROJECT_DIR,"workspace","quarantine/"),
    'datarmor_mpc': os.path.join(PROJECT_DIR_DATARMOR, "workspace", "quarantine/")
}


"""
    Author:  Antoine.Grouazel@ifremer.fr
    Purpose:separate information in SAFE name sentinel1
    Creation:  2014-11-28
    Arguments: basename SAFE directory
    note: valid also for Sentinel3 SRAL data
"""
import sys
import logging
import datetime

fields = ['satellite', 'mode', 'product', 'level', 'polarisation'
    , 'startdate', 'enddate', 'absolute_orbit_number',
          'mission_data_take', 'product_id', 'kind']


class ExplodeSAFE(object):
    '''input basename_safe (str) SAFE name
     only (no parent directories before neitheir children files) '''

    def __init__(self, basename_safe):
        if '/' in basename_safe:
            raise Exception('need basename not full path')
        if basename_safe[0:2] == 'S1':
            self.safename = basename_safe
            self.satellite = self.safename[0:3]
            self.mode = self.safename[4:6]
            self.product = self.safename[7:11]
            self.level = self.safename[12]
            self.kind = self.safename[13]
            self.polarisation = self.safename[14:16]
            self.startdate = datetime.datetime.strptime(self.safename[17:32], '%Y%m%dT%H%M%S')
            self.enddate = datetime.datetime.strptime(self.safename[33:48], '%Y%m%dT%H%M%S')
            self.absolute_orbit_number = self.safename[49:55]
            self.duration = (self.enddate - self.startdate).total_seconds()
            self.sensor = 'CbandRadar'
            self.mission_data_take = self.safename[56:62]  # datatake id
            self.product_id = self.safename[
                              63:67]  # unique id (processing ID) for a given product id( you can have the same for different product_id)
            self.production_status = "operational"
            self.cycle_number = None
            self.relative_orbit_number = None

        elif basename_safe[0:2] == 'S3':
            self.safename = basename_safe
            splitos = self.safename.split('_')
            self.satellite = splitos[0]
            self.mode = None
            self.sensor = splitos[1]
            self.product = splitos[3]
            self.duration = splitos[10]
            self.level = splitos[2]
            self.kind = None
            #             self.kind = self.safename[13]
            self.polarisation = None
            #             print splitos[7]
            self.startdate = datetime.datetime.strptime(splitos[7], '%Y%m%dT%H%M%S')
            self.enddate = datetime.datetime.strptime(splitos[9], '%Y%m%dT%H%M%S')
            self.absolute_orbit_number = None
            self.cycle_number = splitos[11]
            self.relative_orbit_number = splitos[12]
            self.mission_data_take = splitos[9]  # product id
            self.product_generating_center = splitos[18]
            self.product_id = splitos[
                10]  # unique id for a given product id( you can have the same for different product_id)
            productions_status_code = {'O': 'operational',
                                       'F': 'reference',
                                       'D': 'development',
                                       'R': 'reprocessing'
                                       }
            self.production_status = productions_status_code[splitos[19]]
        return

    def props(self):
        return [i for i in self.__dict__.keys() if i[:1] != '_']

    def get(self, info):
        res = getattr(self, info)
        return res


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    if len(sys.argv) > 1:
        safe = sys.argv[1]
    else:
        #     safe = 'S1A_WV_SLC__1SSV_20141113T141141_20141113T143623_003264_003C69_1CDB.SAFE'
        safe = 'S3A_SR_2_WAT____20170124T120058_20170124T121058_20170124T140548_0599_013_294______MAR_O_NR_002.SEN3'  # attention fichiers coupe en demi orbit mais une seul numero de cycle
    logging.info('%s', safe)
    obj = ExplodeSAFE(safe)
    print(obj.get('startdate'))
    #     for ff in fields:
    for ff in obj.props():
        val = obj.get(ff)
        logging.debug('info %s => %s', ff, val)
    print('start date=', obj.startdate)



#!/usr/bin/python
import os
import logging
import get_path_from_base_SAFE
import glob
from explodesafename import ExplodeSAFE
import datetime


def match_GRD_SLC(safenamegrd):
    if 'GRDH' in safenamegrd:
        res = safenamegrd.replace('GRDH', 'SLC_')
    return res


def match_SLC_GRD(safenameslc, type_seek='GRDH'):
    """
    return the safe if it exists at ifremer otherwise None
    """
    goodsafe = None
    mini_ecart = datetime.timedelta(seconds=500000000)
    res = safenameslc.replace('SLC_', type_seek)
    obj = ExplodeSAFE(res)
    st = obj.get('startdate')
    res_base = res[0:10] + '*.SAFE'
    logging.debug('res: %s', res)
    fp = get_path_from_base_SAFE.get_path_from_base_SAFE(res)
    base = os.path.basename(fp)
    fp = fp.replace(base, res_base)
    pot = glob.glob(fp)
    for safe in pot:
        logging.debug('potential safe %s', safe)
        basesafel1 = os.path.basename(safe)
        instance = ExplodeSAFE(basesafel1)
        l1st = instance.get('startdate')
        l1ee = instance.get('enddate')
        #         if l1st<=st and l1ee>=ee:
        if abs(st - l1st) < mini_ecart:
            mini_ecart = abs(st - l1st)
            if mini_ecart < datetime.timedelta(seconds=5):
                goodsafe = safe
    return goodsafe



"""
@author: agrouaze
#oct 2014
@purpose: from a safe filename it gives the right path to the ESL FTP dir
"""
import os
import datetime
import logging
from explodesafename import ExplodeSAFE
from shared_information import datarmor_archive_esa_ifremer, QUARANTINE as quarantine_s1

ADDITIONAL_ARCHIVES = {
    'mpc': '/home/cercache/project/mpc-sentinel1/data/esa/',
    'swarp': '/home/datawork-cersat-public/project/mpc-sentinel1/data/out_of_cal_sites_interesting_products/swarp_arctic/',
    'benguela': '/home/datawork-cersat-public/project/mpc-sentinel1/data/out_of_cal_sites_interesting_products/upwelling_benguela/',
    'canaries': '/home/datawork-cersat-public/project/mpc-sentinel1/data/out_of_cal_sites_interesting_products/upwelling_canaries/',
    #                        's3sral':'/home/cerdata/provider/eumetsat/satellite/l2/sentinel-3/sral/data'
    's3sral': '/home/datawork-cersat-public/archive/provider/eumetsat/satellite/l2/sentinel-3/sral/data',
    "datarmor_mpc": datarmor_archive_esa_ifremer
}
WORKING_DIR = {'mpc': '/home/cercache/project/mpc-sentinel1/workspace',
               'datarmor_mpc': '/home/datawork-cersat-public/cache/project/mpc-sentinel1/workspace'
               }
SPOOL_REP = {  # deprecated
    'mpc': 'spool/',
    'datarmor_mpc': 'spool_datarmor/',
    'benguela': 'spool_benguela/',
    'canaries': 'spool_canaries/',
    'swarp': 'spool_swarp/',
}


def WhichWorkingDir(archive):
    """

    :param archive:
    :return:
    """
    res = WORKING_DIR[archive]
    return res


def WhichFTPdir(safe):
    '''
    input example : S1A_IW_RAW__0SDV_20140526T145627_20140526T145655_000770_000BAE_BC53.SAFE
    '''
    satellite = safe[0:3]
    url = '/data/' + satellite + '/'
    acqui = safe.split('_')[1]
    subprod = safe[0:14]
    subdir = satellite + '_' + acqui + '/'
    finalurl = os.path.join(url, subdir, subprod, 'unsorted/')
    return finalurl


def WhichArchiveDir(safe):
    """
    Args:
        safe (str): safe base name
    """
    gooddir = None
    satdir = 'unknown'
    if safe[0:2] == 'S1':
        firstdate = safe[17:25]
        year = firstdate[0:4]
        doy = str(datetime.datetime.strptime(firstdate, '%Y%m%d').timetuple().tm_yday).zfill(3)
        sat = safe.split('_')[0]
        if sat == 'S1A':
            satdir = 'sentinel-1a'
        elif sat == 'S1B':
            satdir = 'sentinel-1b'
        else:
            logging.error('%s is not a  good satellite name', sat)
        acqui = safe.split('_')[1]
        if acqui[0] == 'S':
            acqui = 'SM'
        level = safe[12:13]
        subproddir = 'L' + level
        datatype = safe[0:14]
        logging.debug('dattype: %s', datatype)
        repdata = WhichArchive_datatype(datatype)
        subname = safe[6:14]
        litlerep = sat + '_' + acqui + subname
        gooddir = os.path.join(repdata, satdir, subproddir, acqui, litlerep, year, doy + '/')
    elif safe[0:2] == 'S3':  # sentinel-3 case
        inst = ExplodeSAFE(safe)
        year = inst.startdate.strftime('%Y')
        doy = inst.startdate.strftime('%j')
        gooddir = os.path.join(ADDITIONAL_ARCHIVES['s3sral'], year, doy)
    return gooddir


def WhichArchive_datatype(datatype):
    """
    datatype (str): ex S1A_IW_GRDH_1S
    version 1.0: 18dec2017, only images mode level1 and lvel0 are stored on datarmor
    """
    acqui = datatype.split('_')[1]
    level = datatype[-2]
    if level == '0' or level == '1' or (level == '2' and acqui == 'EW') or (level == '2' and acqui == 'IW') or (
            level == '2' and acqui == 'SM'):  # since 19 march 2018 #level EW-IW-SM 15nov2018
        repdata = ADDITIONAL_ARCHIVES['datarmor_mpc']
    else:
        repdata = ADDITIONAL_ARCHIVES['datarmor_mpc']  # turned into datarmor in Dec 2019
    return repdata


def WhichSpoolDir(safe=None, archive='datarmor_mpc'):
    """
    Args:
        safe (str): safe basename with .SAFE extension
        archive (str):
    """
    if safe is None:
        spooldir = os.path.join(WhichWorkingDir(archive), SPOOL_REP[archive])
    else:
        if safe[0:2] == 'S1':
            spooldir = os.path.join(WhichWorkingDir(archive), SPOOL_REP[archive])
            spooldir = os.path.join(spooldir)  # change this to get a real spool where the product can be drop easily
        else:
            raise Exception('safe %s doesnt start with S1' % safe)
    return spooldir


def whichquarantinedir(archive='datarmor_mpc'):
    """
    Args:
        archive (str):
    """
    res = quarantine_s1[archive]
    return res


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test = 'S1A_IW_RAW__0SDV_20140526T145627_20140526T145655_000770_000BAE_BC53.SAFE'
    print(test)
    print("spool", WhichSpoolDir(test))
    print("ftp", WhichFTPdir(test))
    tmp = WhichArchiveDir(test)
    print("archive", tmp)



import glob
import os
import panel as pn
import logging
from bokeh.plotting import figure
from iwslcxspecapp.iw_slc_xspec_visu_panel_app import monAppIW_SLC,small_plot_width,\
    small_plot_height,checkbox_burst,checkbox_subswath
import yaml
# Read YAML file
src_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)),'iwslcxspecapp'))
if os.path.exists(os.path.join(src_dir,"localconfig.yml")):
    config_path = os.path.join(src_dir,"localconfig.yml")
else:
    config_path = os.path.join(src_dir,"config.yml")

print('config_path',config_path)
with open(config_path, 'r') as stream:
    data_loaded = yaml.safe_load(stream)
# L1B_file_default = "/home1/scratch/agrouaze/l1b/S1A_IW_SLC__1SDV_20180104T061957_20180104T062025_020001_02211F_77E6.SAFE_L1B_xspec_IFR_VV_0.1.nc"
# L1B_file_default = '/home1/scratch/agrouaze/l1b/S1A_IW_SLC__1SDV_20210413T043041_20210413T043109_037427_04695F_FE5C.SAFE_L1B_xspec_IFR_0.3.nc'
# L1B_file_default = '/home1/scratch/agrouaze/l1b/S1A_IW_SLC__1SDV_20220921T063225_20220921T063252_045099_05639C_D284.SAFE/s1a-iw2-slc-vv-20220921t063225-20220921t063251-045099-05639c-005_L1B_xspec_IFR_0.6.nc'
# L1B_file_default = '/home1/scratch/agrouaze/l1b/S1A_IW_SLC__1SDV_20220921T063225_20220921T063252_045099_05639C_D284.SAFE/s1a-iw2-slc-vv-20220921t063225-20220921t063251-045099-05639c-005_L1B_xspec_IFR_0.6.nc'
# L1B_file_default = '/home1/scratch/agrouaze/l1b/S1A_IW_SLC__1SDV_20170907T103019_20170907T103047_018268_01EB76_5F55.SAFE/s1a-iw2-slc-vv-20170907t103020-20170907t103045-018268-01eb76-005_L1B_xspec_IFR_0.6.nc'
# subswath = 'subswath_1'
#subswath = 'iw1_vv'
L1B_file_default = data_loaded['datadir']
files_dir = data_loaded['files_dir']
if files_dir == 'None':
    files_dir = os.path.abspath(os.path.join(src_dir,'..','assets'))
print('L1B_file_default',L1B_file_default)



############################################################################
#####################NO MAIN BUT IT LOOKS LIKE#############################
logging.basicConfig(level=logging.INFO,force=True)

xsrehandler1 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                      tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
# xsrehandler1 = display_xspec_cart_holo(ds_intra,bu = 3,li = 0,sam = 4,typee='Re')
xsimhandler1 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                      tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
# xsimhandler1 = display_xspec_cart_holo(ds,bu = 3,li = 0,sam = 4,typee='Im')
rough_handler1 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                        tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")

xsrehandler2 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                      tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
# xsrehandler2 = display_xspec_cart_holo(ds,bu = 3,li = 0,sam = 5,typee='Re')
xsimhandler2 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                      tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
# xsimhandler2 = display_xspec_cart_holo(ds,bu = 3,li = 0,sam = 5,typee='Im')
rough_handler2 = figure(plot_height=small_plot_height, plot_width=small_plot_width,
                        tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")

# button intra / inter

streams_select = dict(burst_type=checkbox_burst.param.value)

#files_dir = os.path.abspath(os.path.join(src_dir,'..','assets','S1*','s1*L1B_xspec_IFR*.nc'))
pattern_list_files_dir = os.path.abspath(os.path.join(files_dir,'S1*','s1*L1B_xspec_IFR*.nc'))
print('files_dir',files_dir)
print('pattern_list_files_dir',pattern_list_files_dir)
all_avail_l1B = sorted(glob.glob(pattern_list_files_dir))
print('all available L1B', len(all_avail_l1B))

# files = pn.widgets.FileSelector('/home1/scratch/agrouaze/l1b/',only_files=True,file_pattern='S1*L1B_xspec_IFR*.nc') # ,value=[L1B_file_default]
# logging.info('defaut component defined')
# widget_dmap = hv.DynamicMap(update_app_burst, streams=streams_select)
# widget_dmap = pn.bind(update_app_burst,burst_type=checkbox.param.value)
instclass = monAppIW_SLC()

layout = pn.Row(pn.bind(instclass.update_app_burst,
                        burst_type=checkbox_burst.param.value,
                        subswath_id=checkbox_subswath.param.value,
                        L1B_file=instclass.get_checkboxes(all_avail_l1B).param.value,
                        all_avail_l1B = all_avail_l1B,
                        ))
# pn.Row(widget_dmap).servable()
# layout = pn.Row(xsrehandler1)
print('go servable')
layout.servable()
# pn.panel("# Test").servable()



"""
author: Antoine Grouazel
"""
import sys
import os
from SAFEsortingfunctions import WhichArchiveDir


def get_safe_basename_from_fullpath_measu(fullpathmeasu):
    basename_safe = os.path.basename(os.path.abspath(os.path.join(fullpathmeasu, os.path.pardir, os.path.pardir)))
    return basename_safe


def get_path_from_base_SAFE(inputa):
    if '.SAFE' not in inputa:
        inputa += '.SAFE'
    dira = WhichArchiveDir(inputa)
    final_path = os.path.join(dira, inputa)
    return final_path


if __name__ == '__main__':
    # input is supposed to be a SAFE "S1A....SAFE"
    inputa = sys.argv[1]
    final_path = get_path_from_base_SAFE(inputa)
    print(final_path)



"""
A Grouazel
SARWAVE L1B IW SLC product exploration (visu xspectra inter and intra)
usage:
conda activate py3_panel
 panel serve iw_slc_xspec_visu_panel_app.py --allow-websocket-origin=hostname:5006
"""
import panel as pn
import numpy as np
import holoviews as hv
import datatree
import xarray as xr
import logging
import geopandas as gpd
import copy
import hvplot.pandas
import hvplot.xarray  # noqa
import cartopy.crs as ccrs
import geoviews as gv
import xsar
from shapely.ops import transform
from shapely.geometry import Point
import pyproj
import sys
# import iwslcxspecapp.get_path_from_base_SAFE
# import iwslcxspecapp.match_GRD_SLC
from iwslcxspecapp.symmetrize_xspec import symmetrize_xspectrum
import os
import glob
from cartopy import crs
from holoviews.operation.datashader import datashade, rasterize
from matplotlib import colors as mcolors
import pandas as pd
from bokeh.models import (CDSView, ColorBar, ColumnDataSource,
                          CustomJS, CustomJSFilter,
                          GeoJSONDataSource, HoverTool,
                          LinearColorMapper, Slider)
from bokeh.plotting import figure
webMercator = pyproj.CRS('EPSG:3857')
main_map_width = 600
main_map_height = 500
small_plot_height = 300
small_plot_width = 400
checkbox_burst = pn.widgets.Select(options=['intra', 'inter'], name='burst Type')
checkbox_subswath = pn.widgets.Select(options=['iw1_vv', 'iw2_vv', 'iw3_vv', 'iw1_vh'], name='subswath #')


################################################
def get_tile_corner_image_idx(ds, bursti, tile_line_i, tile_sample_i):
    """

    Parameters
    ----------
    ds
    bursti
    tile_line_i
    tile_sample_i

    Returns
    -------

    """
    top_line = ds['line'].isel({"burst": bursti, "tile_line": tile_line_i}).values + ds.tile_nperseg_line / 2
    logging.debug('ds[sample] %s', ds['sample'])
    left_sample = ds['sample'].isel({"burst": bursti, "tile_sample": tile_sample_i}).values - ds.tile_nperseg_sample / 2
    # left_sample = ds['sample'].isel({"tile_sample": tile_sample_i}).values - ds.tile_nperseg_sample / 2
    bottom_line = ds['line'].isel({"burst": bursti, "tile_line": tile_line_i}).values - ds.tile_nperseg_line / 2
    # bottom_line = ds['line'].isel({"tile_line": tile_line_i}).values - ds.tile_nperseg_line / 2
    right_sample = ds['sample'].isel(
        {"burst": bursti, "tile_sample": tile_sample_i}).values + ds.tile_nperseg_sample / 2
    # right_sample = ds['sample'].isel(
    #    {"tile_sample": tile_sample_i}).values + ds.tile_nperseg_sample / 2

    lines = [top_line, top_line, bottom_line, bottom_line, top_line]
    samples = [left_sample, right_sample, right_sample, left_sample, left_sample]
    return lines, samples


def read_data_L1B(L1B_file, typee='intra'):
    # parentdir = os.path.basename(os.path.dirname(L1B_file))
    #dt = datatree.open_datatree(L1B_file)
    ds = xr.open_dataset(L1B_file, group=typee+'burst')
    # if typee == 'intra':
    #     # ds = dt[subswath]['intraburst_xspectra'].to_dataset()
    #     #ds = dt['intraburst'].to_dataset()
    #
    # else:
    #     # ds = dt[subswath]['interburst_xspectra'].to_dataset()
    #     #ds = dt['interburst'].to_dataset()
    return ds


#
def get_tabulated_burst_data(ds):
    """

    Parameters
    ----------
    ds: xarr.Dataset L1B

    Returns
    -------

    """
    allons = []
    allats = []
    altileline = []
    altilesample = []
    alburst = []
    logging.debug('start tabulation L1B')
    #print('ds["corner_longitude"]', ds['corner_longitude'])
    for iburst in range(ds.burst.size):
        for itilesample in range(ds['corner_longitude'].tile_sample.size):
            for itileline in range(ds['corner_longitude'].tile_line.size):
                allons.append(
                    ds['longitude'].isel({'burst': iburst, 'tile_line': itileline, 'tile_sample': itilesample}).values)
                allats.append(
                    ds['latitude'].isel({'burst': iburst, 'tile_line': itileline, 'tile_sample': itilesample}).values)
                altileline.append(itileline)
                altilesample.append(itilesample)
                alburst.append(iburst)
    # cds = ColumnDataSource(pd.DataFrame({'longitude':allons,'latitude':allats,'Tsample':altilesample,
    #                                      'Tline':altileline,'burst':alburst}))
    # cds = hv.Dataset(data={'longitude':allons,'latitude':allats,'Tsample':altilesample,'Tline':altileline,'burst':alburst})
    cds = pd.DataFrame(
        {'longitude': allons, 'latitude': allats, 'Tsample': altilesample, 'Tline': altileline, 'burst': alburst})
    return cds


#
#

#

#
#
def add_cartesian_wavelength_circles(default_values=[100, 300, 600]):
    """

    Parameters
    ----------
    default_values list of ntegers

    Returns
    -------

    """

    def cart2pol(x, y):
        rho = np.sqrt(x ** 2 + y ** 2)
        phi = np.arctan2(y, x)
        return (rho, phi)

    def pol2cart(rho, phi):
        x = rho * np.cos(phi)
        y = rho * np.sin(phi)
        return (x, y)

    N_pts = 100
    phi = np.linspace(0, 2 * np.pi, N_pts)
    res = {}
    for rru in default_values:
        r100 = np.ones(N_pts) * 2 * np.pi / rru
        coords_cart_100 = []
        for uu in range(N_pts):
            coords_cart_100.append(pol2cart(r100[uu], phi[uu]))
        coords_cart_100 = np.array(coords_cart_100)
        res[rru] = coords_cart_100
    all_circles = []
    for cck in res:
        ccv = res[cck]
        ccim = hv.Curve((ccv[:, 0], ccv[:, 1]), label='%s m' % cck)
        all_circles.append(ccim)
    return hv.Overlay(all_circles)


#
#
def display_xspec_cart_holo(ds, bu=0, li=0, sam=0, typee='Re'):
    """

    Parameters
    ----------
    ds: dataset xarray with xspectra L1B IW intra or inter burst for one subswath
    bu: int
    li: int
    sam: int
    typee :str

    Returns
    -------

    """
    logging.debug('start display_xspec_cart_holo')
    if typee == 'Re':
        cmap = mcolors.LinearSegmentedColormap.from_list("", ["white", "violet", "mediumpurple", "cyan", "springgreen",
                                                              "yellow", "red"])
    else:
        cmap = 'PuOr'

    # if False:
    #     set_xspec = set_xspec.assign_coords({'ky': set_xspec['k_az'], 'kx': set_xspec['k_rg']})
    #     set_xspec = set_xspec.swap_dims({'freq_line': 'ky', 'freq_sample': 'kx'})
    #     kx_varname = 'kx'
    #     ky_varname = 'ky'
    # else:
    # gather Re and Imaginary part
    if 'xspectra_0tau_Re' in ds: #intra burst case
        for tautau in range(3):
            ds['xspectra_%stau' % tautau] = ds['xspectra_%stau_Re' % tautau] + 1j * ds['xspectra_%stau_Im' % tautau]
            ds = ds.drop(['xspectra_%stau_Re' % tautau, 'xspectra_%stau_Im' % tautau])
    else: # inter burst case
        ds['xspectra'] = ds['xspectra_Re'] + 1j * ds['xspectra_Im']
        ds = ds.drop(['xspectra_Re', 'xspectra_Im'])
    set_xspec = ds[{'burst': bu, 'tile_line': li, 'tile_sample': sam}]
    set_xspec = set_xspec.swap_dims({'freq_line': 'k_az', 'freq_sample': 'k_rg'})
    logging.debug('symmetrize_xspectrum')
    set_xspec = symmetrize_xspectrum(set_xspec, dim_range='k_rg', dim_azimuth='k_az')
    logging.debug('symmetrzied')
    kx_varname = 'k_rg'
    ky_varname = 'k_az'
    # sp = set_xspec['xspectra_2tau_%s' % typee].mean(dim='2tau')
    if 'xspectra_2tau' in set_xspec:
        sp = set_xspec['xspectra_2tau'].mean(dim='2tau')
    else:
        sp = set_xspec['xspectra'] # inter burst case
    sp2 = sp.where(np.logical_and(np.abs(sp[kx_varname]) <= 0.14, np.abs(sp[ky_varname]) <= 0.14), drop=True)
    logging.debug('spectra is ready')
    hv.extension('bokeh')
    if typee == 'Re':
        im = hv.Image(abs(sp2.real), kdims=[kx_varname, ky_varname]).opts(width=small_plot_width,
                                                                          height=small_plot_height,
                                                                          cmap=cmap, colorbar=True,
                                                                          xlim=(    -0.07, 0.07),
                                                                          ylim=(-0.07, 0.07),
                                                                          )  # gist_ncar_r , 'cet_linear_wyor_100_45_c55'
    else:
        if np.all(np.isnan(sp2.imag)):
            extrema = 1
        else:
            extrema = float(abs(sp2.imag).max().values)

        im = hv.Image(sp2.imag, kdims=[kx_varname, ky_varname]).opts(width=small_plot_width,
                                                                     height=small_plot_height,
                                                                     cmap=cmap, colorbar=True,
                                                                     xlim=(-0.07, 0.07),
                                                                     ylim=(-0.07, 0.07),
                                                                     show_grid=True, clim=(-extrema, extrema))
    tt = hv.Text(-0.06, 0.05, 'burst %s\ntile range: %s\ntile azimuth: %s' % (bu, sam, li), halign='left', fontsize=8)
    logging.debug('add circles on plot')
    cc = add_cartesian_wavelength_circles(default_values=[100, 300, 600])
    return im * tt * cc
#


class monAppIW_SLC:

    def __init__(self):
        self.previous_xspec_selected = 0
        self.latest_click = 0
        # prepare data
        logging.info('ok init')
        self.ds_intra = None
        self.cds_intra = None
        self.ds_inter = None
        self.cds_inter = None
        self.l1bpath = None
        self.subswath = None
        self.xsarobjslc = None
        self.xsarobjgrd = None
        self.burst_type = 'intra'
        self.lons =  None
        self.lats = None



    def display_roughness_slc(self, l1b_path, subswath, burst, tile_sample_id, tile_line_id, dsl1b):
        """

        Parameters
        ----------
        l1b_path: str
        subswath: str subswath_2 for example
        burst: int
        tile_sample_id: int
        tile_line_id: int
        dsl1b: xarray.Dataset  L1B

        Returns
        -------

        """
        logging.debug('start roughness display')
        lines, samples = get_tile_corner_image_idx(dsl1b, bursti=burst,
                                                   tile_line_i=tile_line_id,
                                                   tile_sample_i=tile_sample_id)
        logging.debug('lines %s', lines)
        lines = np.array(lines).astype(int)
        samples = np.array(samples).astype(int)
        start_line = lines.min()
        stop_line = lines.max()
        start_sample = samples.min()
        stop_sample = samples.max()
        rough = abs(self.xsarobjslc.dataset['digital_number'].isel(
            {"line": slice(start_line, stop_line), 'sample': slice(start_sample, stop_sample), 'pol': 0}))
        p95percentile = np.percentile(rough.values.squeeze(), 95)
        return rasterize(hv.Image(rough, kdims=['sample', 'line']).opts(cmap='gray', colorbar=True, tools=['hover'],
                                                                        title="%s" % rough.pol.values, width=400,
                                                                        height=300, clim=(0, p95percentile),
                                                                        shared_axes=False))  #

    def display_roughness_grd(self):
        """
        return the nice display from GRD product
        Parameters
        ----------
        l1b_path: str of SLC product
        burst: int
        tile_sample_id: int
        tile_line_id: int
        dsl1b: xarray.Dataset  L1B

        Returns
        -------

        """
        rough = self.xsarobjgrd.dataset['sigma0'].rio.reproject('epsg:4326',
                                                                shape=(1000, 1000), nodata=np.nan).isel({'pol': 0})
        logging.debug(rough)
        # rough = abs(self.xsarobjgrd.dataset['sigma0'].isel({ 'pol': 0})).values.ravel()

        p95percentile = np.nanpercentile(rough.squeeze().values.ravel(), 95)
        logging.debug('p95percentile GRD %s', p95percentile)
        # lons = self.xsarobjgrd.dataset['longitude'].values.ravel()
        # lats = self.xsarobjgrd.dataset['latitude'].values.ravel()
        # print('lats,',lats.shape)
        # data3d = np.stack([lons,lats,rough]).T
        # print('data3d;shape',data3d.shape)
        # res = rasterize(hv.Scatter(data3d,kdims=['longitude', 'latitude']
        #                             ).opts(cmap='gray', colorbar=True, tools=['hover'],
        #                             title="%s" % self.xsarobjgrd.dataset['sigma0'].pol.values, width=400,
        #                             height=300, clim=(0, p95percentile),shared_axes=False,alpha=4,color='z'),
        #                  )  #kdims=['longitude', 'latitude']  vdims=['longitude', 'latitude', 'z']
        # res = hv.Image(rough).opts(cmap='gray', colorbar=False, tools=[],
        #                             title="%s" % rough.pol.values, width=main_map_width,
        #                             height=main_map_height,shared_axes=False,alpha=0.4,clim=(0, p95percentile))*gv.Path(self.xsarobjgrd.footprint)

        # rough2 = rough.assign_coords({'longitude':'x','latitude':'y'})
        rough2 = rough.assign_coords({
            'Longitude': xr.DataArray(rough['x'].values, dims=['x']),
            'Latitude': xr.DataArray(rough['y'].values, dims=['y']),
        }
        )
        rough2 = rough2.swap_dims({'x': 'Longitude', 'y': 'Latitude'})
        new_ds = xr.Dataset()
        new_ds['s0grd'] = rough2
        # res = gv.Image(rough2,label='grd').opts(cmap='gray', colorbar=False, tools=[],
        #                             title="%s" % rough.pol.values, width=main_map_width,show_legend=True,
        #                             height=main_map_height,shared_axes=True,alpha=0.4,clim=(0, p95percentile))*gv.Path(self.xsarobjgrd.footprint)
        # res = new_ds.hvplot.image(x='Longitude',y='Latitude',z="s0grd") # OK but not more tap tool
        res = gv.Path(self.xsarobjgrd.footprint)
        return res

    def set_input_l1b_data(self,L1B_file,burst_type):
        self.l1bpath = L1B_file
        subswath_id = os.path.basename(L1B_file).split('-')[1] + '_' + os.path.basename(L1B_file).split('-')[3]
        self.subswath = subswath_id
        self.burst_type = burst_type
        self.ds_intra = read_data_L1B(L1B_file, typee='intra')
        self.cds_intra = get_tabulated_burst_data(self.ds_intra)
        self.ds_inter = read_data_L1B(L1B_file, typee='inter')
        self.cds_inter = get_tabulated_burst_data(self.ds_inter)

    @pn.depends(checkbox_burst.param.value)
    def reset_index_xpsec_selected(self):
        """
        index of points selected are integer in the order of vectorized tabulated data
        :return:
        """
        self.previous_xspec_selected = 0
        self.latest_click = 0


    def update_app_burst(self,burst_type=None, subswath_id=None, L1B_file=None, all_avail_l1B=None):
        """

        Parameters
        ----------
        burst_type:  str intra or inter
        subswath_id: str iw1
        L1B_file: str one subswath .nc file (previous version it was one nc file for a SAFE)
        all_avail_l1B : list

        Returns
        -------

        """
        logging.info('update_app_burst')
        self.display_rough = False  # tmp swith off for local test, march 2023
        # if L1B_file==[]:
        #     L1B_file = L1B_file_default #security util???
        if isinstance(L1B_file, list):
            if len(L1B_file) != 0:
                L1B_file = L1B_file[0]
                logging.debug('new L1B %s ', L1B_file)
            else:
                pass
                # L1B_file = L1B_file_default  # security util???
        # prepare data
        logging.info('oui')

        logging.debug('L1B_file %s', L1B_file)
        if L1B_file != self.l1bpath or subswath_id != self.subswath or burst_type != self.burst_type:
            logging.debug('go for reading L1B')
            self.set_input_l1b_data(L1B_file,burst_type)

            logging.debug('ok data is loaded')
            if self.display_rough:
                # base = os.path.basename(self.l1bpath).split('_L1B')[0]
                base = os.path.basename(os.path.dirname(self.l1bpath))
                logging.debug('base %s', base)
                # fullpath_safeL1SLC = get_path_from_base_SAFE.get_path_from_base_SAFE(base)
                fullpath_safeL1SLC = os.path.join(os.path.dirname(os.path.dirname(L1B_file)), 'raw_data', base)
                logging.debug('fullpath_safeL1SLC %s', fullpath_safeL1SLC)
                if subswath_id is not None:
                    subswath_nb = subswath_id.split('_')[0][-1]
                str_gdal = 'SENTINEL1_DS:%s:IW%s' % (fullpath_safeL1SLC, subswath_nb)
                logging.debug('str_gdal %s', str_gdal)
                self.xsarobjslc = xsar.Sentinel1Dataset(str_gdal)  # ,resolution='10m'
                grdh_path = match_GRD_SLC.match_SLC_GRD(os.path.basename(fullpath_safeL1SLC), type_seek='GRDH')
                if grdh_path:
                    self.xsarobjgrd = xsar.Sentinel1Dataset(grdh_path, resolution='200m')  # ,resolution='10m'
                else:
                    logging.debug('impossible to have GRD product')
        else:
            logging.debug('nothing to do')

        #####
        self.maphandler = figure(x_range=(-19000000, 8000000), y_range=(-1000000, 7000000),
                         x_axis_type="mercator", y_axis_type="mercator", plot_height=800,
                         plot_width=950, tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover,tap")
        hover = self.maphandler.select(dict(type=HoverTool))
        # hover.tooltips = tooltips
        self.maphandler.xgrid.grid_line_color = None
        self.maphandler.ygrid.grid_line_color = None
        # tile_provider = get_provider(CARTODBPOSITRON)
        # maphandler.add_tile(tile_provider)
        if burst_type == 'intra':
            ds = self.ds_intra
            cds = self.cds_intra
        else:
            ds = self.ds_inter
            cds = self.cds_inter
        logging.debug('start grid display')
        self.maphandler = self.display_intra_inter_burst_grids()

        if self.xsarobjgrd:
            logging.debug('display rougness grid')
            self.maphandler = self.display_roughness_grd() * self.maphandler  # cannot click on the point of the xspec grid
            # maphandler = maphandler*self.display_roughness_grd()
            pass
        posxy = hv.streams.Tap(source=self.maphandler, x=0, y=0)
        logging.debug('posxy %s %s %s',posxy,type(posxy),dir(posxy))
        # interactive selection of a point on the map
        # Declare Tap stream with heatmap as source and initial values





        # Connect the Tap stream to the tap_histogram callback
        # tap_dmap = hv.DynamicMap(tap_update_xspec_figures, streams=[posxy, checkbox])
        logging.debug('creating rows and columns for layout panel/bokeh')
        logging.debug('posxy.param.x %s %s %s', posxy.param.x, type(posxy.param.x), dir(posxy.param.x))
        layout_figures = pn.Row(pn.bind(self.tap_update_xspec_figures, x=posxy.param.x,
                                        y=posxy.param.y))


        checkbox_files = self.get_checkboxes(all_avail_l1B=all_avail_l1B)
        bokekjap = pn.Row(
            pn.Column(pn.Column(checkbox_files, pn.Row(checkbox_burst, checkbox_subswath),
                                self.maphandler)),
            layout_figures,
        )
        # bokekjap = pn.Row(
        #     pn.Column(pn.Column(checkbox_files, bindo_burst,pn.Row(checkbox_subswath),
        #                         self.maphandler, posxy)),
        #     layout_figures,
        # )
        logging.debug('return bokeh app layout')
        return bokekjap

    def tap_update_xspec_figures(self,x, y):
        """
        :param y:
        :return:
        """
        if self.burst_type == 'intra':
            ds = self.ds_intra
            cds = self.cds_intra
        else:
            ds = self.ds_inter
            cds = self.cds_inter

        project = pyproj.Transformer.from_crs(webMercator, self.projection, always_xy=True).transform

        xygeo = transform(project, Point(x,y))
        logging.debug('xny %s %s',x,y)
        logging.debug('xygeo, %s %s %s',xygeo, xygeo.x, xygeo.y)

        selected_pt = np.argmin((xygeo.x - self.lons) ** 2 + (xygeo.y - self.lats) ** 2)
        self.previous_xspec_selected = copy.copy(self.latest_click)
        self.latest_click = selected_pt
        logging.debug('selected_pt %s', selected_pt)
        burst = cds['burst'].iloc[selected_pt]
        tile_line = cds['Tline'].iloc[selected_pt]
        tile_sample = cds['Tsample'].iloc[selected_pt]
        xsrehandler1 = display_xspec_cart_holo(ds, bu=burst, li=tile_line, sam=tile_sample, typee='Re')
        xsimhandler1 = display_xspec_cart_holo(ds, bu=burst, li=tile_line, sam=tile_sample, typee='Im')
        # rough_handler1 = figure(plot_height=small_plot_height, plot_width=small_plot_with,
        #                         tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
        logging.debug('xspec figures are OK')
        if self.display_rough:
            rough_handler1 = self.display_roughness_slc(self.l1bpath, self.subswath, burst=burst,
                                                        tile_sample_id=tile_sample,
                                                        tile_line_id=tile_line,
                                                        dsl1b=ds)
        else:
            logging.debug('empty roughness figure')
            rough_handler1 = hv.Image((np.random.rand(100, 100)))
            logging.debug('done')

        logging.debug('start to create 2nd set of xspec figures')
        burst_prev = cds['burst'].iloc[self.previous_xspec_selected]
        tile_line_prev = cds['Tline'].iloc[self.previous_xspec_selected]
        tile_sample_prev = cds['Tsample'].iloc[self.previous_xspec_selected]

        xsrehandler2 = display_xspec_cart_holo(ds, bu=burst_prev, li=tile_line_prev, sam=tile_sample_prev,
                                               typee='Re')
        xsimhandler2 = display_xspec_cart_holo(ds, bu=burst_prev, li=tile_line_prev, sam=tile_sample_prev,
                                               typee='Im')
        # rough_handler1 = figure(plot_height=small_plot_height, plot_width=small_plot_with,
        #                         tools="pan, wheel_zoom, box_zoom, reset,lasso_select,hover")
        if self.display_rough:
            rough_handler2 = self.display_roughness_slc(self.l1bpath, self.subswath, burst=burst_prev,
                                                        tile_sample_id=tile_sample_prev,
                                                        tile_line_id=tile_line_prev,
                                                        dsl1b=ds)
        else:
            rough_handler2 = hv.Image(np.random.rand(100, 100))
        res = pn.Column(
            pn.Row(xsrehandler1, xsimhandler1, rough_handler1),
            # layout_1,
            pn.Row(xsrehandler2, xsimhandler2, rough_handler2),
        )
        #another call to re draw the map and change the red circle position
        #self.maphandler = self.display_intra_inter_burst_grids() #-> leads to not clickable map... to debug
        return res

    def get_checkboxes(self, all_avail_l1B):
        ##############################
        # add the checboxes

        checkbox_files = pn.widgets.Select(options=all_avail_l1B, name='file')
        ##############################
        return checkbox_files


    def display_intra_inter_burst_grids(self):
        """

        Parameters
        ----------
        ds dataset xarray one sub-swath intra or inter burst
        cds pandas DataFrame computed from ds
        burst_type str intra or inter

        Returns
        -------

        """
        all_poly = []
        if self.burst_type == 'intra':
            ds = self.ds_intra.load()
            cds = self.cds_intra
        else:
            ds = self.ds_inter.load()
            cds = self.cds_inter
        self.lons = cds['longitude'].values
        self.lats = cds['latitude'].values
        ds['corner_longitude'] = ds['corner_longitude']
        ds['corner_latitude'] = ds['corner_latitude']
        for iburst in range(ds.burst.size):
            for itilesample in range(ds['corner_longitude'].tile_sample.size):
                for itileline in range(ds['corner_longitude'].tile_line.size):
                    clon = ds['corner_longitude'].isel(
                        {'burst': iburst, 'tile_line': itileline,
                         'tile_sample': itilesample}).values.ravel(order='A')
                    clat = ds['corner_latitude'].isel(
                        {'burst': iburst, 'tile_line': itileline,
                         'tile_sample': itilesample}).values.ravel(order='A')
                    clon2 = copy.copy(clon)
                    clat2 = copy.copy(clat)
                    clat2[2] = clat[3]
                    clat2[3] = clat[2]
                    clon2[2] = clon[3]
                    clon2[3] = clon[2]
                    tmpo = np.stack([clon2, clat2]).T
                    tmpo = np.vstack([tmpo, tmpo[0, :]])
                    tmppoly = gv.Path((tmpo[:, 0], tmpo[:, 1]), kdims=['Longitude', 'Latitude'], ).opts(color='grey')
                    all_poly.append(tmppoly)
        logging.debug('polygons are constructed')
        self.projection = ccrs.PlateCarree()
        if self.burst_type == 'intra':
            coco = 'blue'
        else:
            coco = 'red'
        points = cds.hvplot.points(x='longitude', y='latitude', hover_cols='all', use_index=False,
                                   label=self.burst_type, color=coco, geo=True,
                                   crs=self.projection).opts(tools=['hover','tap'],
                                                             size=10,nonselection_alpha=0.1)
        logging.debug('crs %s', points.crs)
        # points = gv.Points(('longitude','latitude'),source=cds).opts(tools=['hover'])
        # gv.tile_sources.Wikipedia  *


        res = (gv.tile_sources.EsriImagery * gv.Overlay(all_poly) * points).opts(width=main_map_width,
                                                                                         height=main_map_height,
                                                                                         show_legend=True,
                                                                                         title=os.path.basename(
                                                                                             self.l1bpath) + '\n' + self.subswath,
                                                                                         fontsize={'title': 8})
        return res

############################################################################
#####################NO MAIN BUT IT LOOKS LIKE#############################
